﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;

namespace ky1001
{
    public class HandMorph: MVRScript {

        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "HandMorphManager";
        public string pluginVersion = "1.1";
        public string pluginDate = "2022-01-10";
        public string pluginDescription = "";
        #endregion

        #region Vars
        protected Atom CurrentAtom;

        public float ltBendValue;
        public float rtBendValue;
        public float liBendValue;
        public float riBendValue;
        public float lmBendValue;
        public float rmBendValue;
        public float lrBendValue;
        public float rrBendValue;
        public float lpBendValue;
        public float rpBendValue;

        public float ltFistValue;
        public float rtFistValue;
        public float ltGraspValue;
        public float rtGraspValue;
        public float ltInOutValue;
        public float rtInOutValue;
        public float ltStraightValue;
        public float rtStraightValue;

        public float lfFistValue;
        public float rfFistValue;
        public float lfGraspValue;
        public float rfGraspValue;
        public float lfInOutValue;
        public float rfInOutValue;
        public float lfSqueezeValue;
        public float rfSqueezeValue;
        public float lfStraightValue;
        public float rfStraightValue;

        public float lhFistValue;
        public float rhFistValue;
        public float lhGraspValue;
        public float rhGraspValue;
        public float lhSpreadValue;
        public float rhSpreadValue;
        public float lhChopValue;
        public float rhChopValue;
        public float lhStraightValue;
        public float rhStraightValue;       

        private const string _saveExt = "handmorph";
        private const string _handmorphSavePath = "Saves\\PluginData\\handmorph";
        public string SaveData;
        public string LoadData;

        protected JSONStorableBool toggle_x;
        protected JSONStorableBool toggle_l;
        protected JSONStorableBool toggle_r;
        #endregion

        private readonly string[] HandMorphs = new string[] {
            "Hand Chop",
            "Hand Fist",
            "Hand Grasp",
            "Hand Spread",
            "Hand Straighten",
            "Fingers Fist",
            "Fingers Grasp",
            "Fingers In-Out",
            "Fingers Squeeze",
            "Fingers Straighten",
            "Thumb Bend",
            "Index Finger Bend",
            "Mid Finger Bend",
            "Ring Finger Bend",
            "Pinky Finger Bend",
            "Thumb Fist",
            "Thumb Grasp",
            "Thumb In-Out",
            "Thumb Straighten",
        };

        private void DefaultHandMorphs()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            List<string> HandMorphList = new List<string>(HandMorphs);
            foreach (string HandMorphs in HandMorphList)
            {
                morphControl.GetMorphByDisplayName("Right "+HandMorphs).morphValue = 0f; 
                morphControl.GetMorphByDisplayName("Left "+HandMorphs).morphValue = 0f; 
            }
            morphControl.GetMorphByDisplayName("Right Fingers Fist").morphValue = 0.15f;
            morphControl.GetMorphByDisplayName("Left Fingers Fist").morphValue = 0.15f;
            morphControl.GetMorphByDisplayName("Right Thumb Fist").morphValue = 0.15f;
            morphControl.GetMorphByDisplayName("Left Thumb Fist").morphValue = 0.15f;
            SliderValueUpdate();
            ChangeSliders();
        }

        private void ZeroHandMorps()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            List<string> HandMorphList = new List<string>(HandMorphs);
            foreach (string HandMorphs in HandMorphList)
            {
                morphControl.GetMorphByDisplayName("Right "+HandMorphs).morphValue = 0f; 
                morphControl.GetMorphByDisplayName("Left "+HandMorphs).morphValue = 0f; 
            }
            SliderValueUpdate();
            ChangeSliders();
        }

        private void GetValue()
        {
            SliderValueUpdate();
            ChangeSliders();            
        }

        private void SliderValueUpdate()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            lhFistSlider.slider.value = morphControl.GetMorphByDisplayName("Left Hand Fist").morphValue; 
            rhFistSlider.slider.value = morphControl.GetMorphByDisplayName("Right Hand Fist").morphValue; 
            lhGraspSlider.slider.value = morphControl.GetMorphByDisplayName("Left Hand Grasp").morphValue; 
            rhGraspSlider.slider.value = morphControl.GetMorphByDisplayName("Right Hand Grasp").morphValue; 
            lhSpreadSlider.slider.value = morphControl.GetMorphByDisplayName("Left Hand Spread").morphValue; 
            rhSpreadSlider.slider.value = morphControl.GetMorphByDisplayName("Right Hand Spread").morphValue; 
            lhChopSlider.slider.value = morphControl.GetMorphByDisplayName("Left Hand Chop").morphValue;
            rhChopSlider.slider.value = morphControl.GetMorphByDisplayName("Right Hand Chop").morphValue;
            lhStraightSlider.slider.value = morphControl.GetMorphByDisplayName("Left Hand Straighten").morphValue; 
            rhStraightSlider.slider.value = morphControl.GetMorphByDisplayName("Right Hand Straighten").morphValue; 

            lfFistSlider.slider.value = morphControl.GetMorphByDisplayName("Left Fingers Fist").morphValue; 
            rfFistSlider.slider.value = morphControl.GetMorphByDisplayName("Right Fingers Fist").morphValue; 
            lfGraspSlider.slider.value = morphControl.GetMorphByDisplayName("Left Fingers Grasp").morphValue; 
            rfGraspSlider.slider.value = morphControl.GetMorphByDisplayName("Right Fingers Grasp").morphValue; 
            lfInOutSlider.slider.value = morphControl.GetMorphByDisplayName("Left Fingers In-Out").morphValue; 
            rfInOutSlider.slider.value = morphControl.GetMorphByDisplayName("Right Fingers In-Out").morphValue; 
            lfSqueezeSlider.slider.value = morphControl.GetMorphByDisplayName("Left Fingers Squeeze").morphValue;
            rfSqueezeSlider.slider.value = morphControl.GetMorphByDisplayName("Right Fingers Squeeze").morphValue;
            lfStraightSlider.slider.value = morphControl.GetMorphByDisplayName("Left Fingers Straighten").morphValue; 
            rfStraightSlider.slider.value = morphControl.GetMorphByDisplayName("Right Fingers Straighten").morphValue; 

            ltBendSlider.slider.value = morphControl.GetMorphByDisplayName("Left Thumb Bend").morphValue; 
            rtBendSlider.slider.value = morphControl.GetMorphByDisplayName("Right Thumb Bend").morphValue; 
            liBendSlider.slider.value = morphControl.GetMorphByDisplayName("Left Index Finger Bend").morphValue; 
            riBendSlider.slider.value = morphControl.GetMorphByDisplayName("Right Index Finger Bend").morphValue; 
            lmBendSlider.slider.value = morphControl.GetMorphByDisplayName("Left Mid Finger Bend").morphValue; 
            rmBendSlider.slider.value = morphControl.GetMorphByDisplayName("Right Mid Finger Bend").morphValue; 
            lrBendSlider.slider.value = morphControl.GetMorphByDisplayName("Left Ring Finger Bend").morphValue; 
            rrBendSlider.slider.value = morphControl.GetMorphByDisplayName("Right Ring Finger Bend").morphValue; 
            lpBendSlider.slider.value = morphControl.GetMorphByDisplayName("Left Pinky Finger Bend").morphValue; 
            rpBendSlider.slider.value = morphControl.GetMorphByDisplayName("Right Pinky Finger Bend").morphValue; 
            ltFistSlider.slider.value = morphControl.GetMorphByDisplayName("Left Thumb Fist").morphValue; 
            rtFistSlider.slider.value = morphControl.GetMorphByDisplayName("Right Thumb Fist").morphValue; 
            ltGraspSlider.slider.value = morphControl.GetMorphByDisplayName("Left Thumb Grasp").morphValue; 
            rtGraspSlider.slider.value = morphControl.GetMorphByDisplayName("Right Thumb Grasp").morphValue; 
            ltInOutSlider.slider.value = morphControl.GetMorphByDisplayName("Left Thumb In-Out").morphValue; 
            rtInOutSlider.slider.value = morphControl.GetMorphByDisplayName("Right Thumb In-Out").morphValue; 
            ltStraightSlider.slider.value = morphControl.GetMorphByDisplayName("Left Thumb Straighten").morphValue; 
            rtStraightSlider.slider.value = morphControl.GetMorphByDisplayName("Right Thumb Straighten").morphValue;         
        }

        private void CopyLtoR()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            List<string> HandMorphList = new List<string>(HandMorphs);
            foreach (string HandMorphs in HandMorphList)
            {
                morphControl.GetMorphByDisplayName("Right "+HandMorphs).morphValue = morphControl.GetMorphByDisplayName("Left "+HandMorphs).morphValue; 
            }
            SliderValueUpdate();
            ChangeSliders();            
        }

        private void CopyRtoL()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            List<string> HandMorphList = new List<string>(HandMorphs);
            foreach (string HandMorphs in HandMorphList)
            {
                morphControl.GetMorphByDisplayName("Left "+HandMorphs).morphValue = morphControl.GetMorphByDisplayName("Right "+HandMorphs).morphValue;
            }
            SliderValueUpdate();
            ChangeSliders();   
        }

        private void SwapLandR()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            List<string> HandMorphList = new List<string>(HandMorphs);
            foreach (string HandMorphs in HandMorphList)
            {
                float l_temp = morphControl.GetMorphByDisplayName("Left "+HandMorphs).morphValue;
                morphControl.GetMorphByDisplayName("Left "+HandMorphs).morphValue = morphControl.GetMorphByDisplayName("Right "+HandMorphs).morphValue;
                morphControl.GetMorphByDisplayName("Right "+HandMorphs).morphValue = l_temp;
            }
            SliderValueUpdate();
            ChangeSliders();   
        }

        private void GetSliderValue()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            lhFistValue = morphControl.GetMorphByDisplayName("Left Hand Fist").morphValue; 
            rhFistValue = morphControl.GetMorphByDisplayName("Right Hand Fist").morphValue; 
            lhGraspValue = morphControl.GetMorphByDisplayName("Left Hand Grasp").morphValue; 
            rhGraspValue = morphControl.GetMorphByDisplayName("Right Hand Grasp").morphValue; 
            lhSpreadValue = morphControl.GetMorphByDisplayName("Left Hand Spread").morphValue; 
            rhSpreadValue = morphControl.GetMorphByDisplayName("Right Hand Spread").morphValue; 
            lhChopValue = morphControl.GetMorphByDisplayName("Left Hand Chop").morphValue;
            rhChopValue = morphControl.GetMorphByDisplayName("Right Hand Chop").morphValue;
            lhStraightValue = morphControl.GetMorphByDisplayName("Left Hand Straighten").morphValue; 
            rhStraightValue = morphControl.GetMorphByDisplayName("Right Hand Straighten").morphValue; 

            lfFistValue = morphControl.GetMorphByDisplayName("Left Fingers Fist").morphValue; 
            rfFistValue = morphControl.GetMorphByDisplayName("Right Fingers Fist").morphValue; 
            lfGraspValue = morphControl.GetMorphByDisplayName("Left Fingers Grasp").morphValue; 
            rfGraspValue = morphControl.GetMorphByDisplayName("Right Fingers Grasp").morphValue; 
            lfInOutValue = morphControl.GetMorphByDisplayName("Left Fingers In-Out").morphValue; 
            rfInOutValue = morphControl.GetMorphByDisplayName("Right Fingers In-Out").morphValue; 
            lfSqueezeValue = morphControl.GetMorphByDisplayName("Left Fingers Squeeze").morphValue;
            rfSqueezeValue = morphControl.GetMorphByDisplayName("Right Fingers Squeeze").morphValue;
            lfStraightValue = morphControl.GetMorphByDisplayName("Left Fingers Straighten").morphValue; 
            rfStraightValue = morphControl.GetMorphByDisplayName("Right Fingers Straighten").morphValue; 

            ltBendValue = morphControl.GetMorphByDisplayName("Left Thumb Bend").morphValue; 
            rtBendValue = morphControl.GetMorphByDisplayName("Right Thumb Bend").morphValue; 
            liBendValue = morphControl.GetMorphByDisplayName("Left Index Finger Bend").morphValue; 
            riBendValue = morphControl.GetMorphByDisplayName("Right Index Finger Bend").morphValue; 
            lmBendValue = morphControl.GetMorphByDisplayName("Left Mid Finger Bend").morphValue; 
            rmBendValue = morphControl.GetMorphByDisplayName("Right Mid Finger Bend").morphValue; 
            lrBendValue = morphControl.GetMorphByDisplayName("Left Ring Finger Bend").morphValue; 
            rrBendValue = morphControl.GetMorphByDisplayName("Right Ring Finger Bend").morphValue; 
            lpBendValue = morphControl.GetMorphByDisplayName("Left Pinky Finger Bend").morphValue; 
            rpBendValue = morphControl.GetMorphByDisplayName("Right Pinky Finger Bend").morphValue; 

            ltFistValue = morphControl.GetMorphByDisplayName("Left Thumb Fist").morphValue; 
            rtFistValue = morphControl.GetMorphByDisplayName("Right Thumb Fist").morphValue; 
            ltGraspValue = morphControl.GetMorphByDisplayName("Left Thumb Grasp").morphValue; 
            rtGraspValue = morphControl.GetMorphByDisplayName("Right Thumb Grasp").morphValue; 
            ltInOutValue = morphControl.GetMorphByDisplayName("Left Thumb In-Out").morphValue; 
            rtInOutValue = morphControl.GetMorphByDisplayName("Right Thumb In-Out").morphValue; 
            ltStraightValue = morphControl.GetMorphByDisplayName("Left Thumb Straighten").morphValue; 
            rtStraightValue = morphControl.GetMorphByDisplayName("Right Thumb Straighten").morphValue; 
        }

        private void CreateSliders()
        {
            JSONStorableFloat lhFist = new JSONStorableFloat("Left Hand Fist", lhFistValue, -1f, 1f, true);
            JSONStorableFloat rhFist = new JSONStorableFloat("Right Hand Fist", rhFistValue, -1f, 1f, true);
            JSONStorableFloat lhGrasp = new JSONStorableFloat("Left Hand Grasp", lhGraspValue, -1f, 1f, true);
            JSONStorableFloat rhGrasp = new JSONStorableFloat("Right Hand Grasp", rhGraspValue, -1f, 1f, true);
            JSONStorableFloat lhSpread = new JSONStorableFloat("Left Hand Spread", lhSpreadValue, -1f, 1f, true);
            JSONStorableFloat rhSpread = new JSONStorableFloat("Right Hand Spread", rhSpreadValue, -1f, 1f, true);
            JSONStorableFloat lhChop = new JSONStorableFloat("Left Hand Chop", lhChopValue, 0f, 1f, true);
            JSONStorableFloat rhChop = new JSONStorableFloat("Right Hand Chop", rhChopValue, 0f, 1f, true);
            JSONStorableFloat lhStraight = new JSONStorableFloat("Left Hand Straighten", lhStraightValue, -1f, 1f, true);
            JSONStorableFloat rhStraight = new JSONStorableFloat("Right Hand Straighten", rhStraightValue, -1f, 1f, true);

            JSONStorableFloat lfFist = new JSONStorableFloat("Left Fingers Fist", lfFistValue, -1f, 1f, true);
            JSONStorableFloat rfFist = new JSONStorableFloat("Right Fingers Fist", rfFistValue, -1f, 1f, true);
            JSONStorableFloat lfGrasp = new JSONStorableFloat("Left Fingers Grasp", lfGraspValue, -1f, 1f, true);
            JSONStorableFloat rfGrasp = new JSONStorableFloat("Right Fingers Grasp", rfGraspValue, -1f, 1f, true);
            JSONStorableFloat lfInOut = new JSONStorableFloat("Left Fingers In-Out", lfInOutValue, -1f, 1f, true);
            JSONStorableFloat rfInOut = new JSONStorableFloat("Right Fingers In-Out", rfInOutValue, -1f, 1f, true);
            JSONStorableFloat lfSqueeze = new JSONStorableFloat("Left Fingers Squeeze", lfSqueezeValue, 0f, 1f, true);
            JSONStorableFloat rfSqueeze = new JSONStorableFloat("Right Fingers Squeeze", rfSqueezeValue, 0f, 1f, true);
            JSONStorableFloat lfStraight = new JSONStorableFloat("Left Fingers Straighten", lfStraightValue, -1f, 1f, true);
            JSONStorableFloat rfStraight = new JSONStorableFloat("Right Fingers Straighten", rfStraightValue, -1f, 1f, true);

            JSONStorableFloat ltBend = new JSONStorableFloat("Left Thumb Bend", ltBendValue, -1f, 1f, true);
            JSONStorableFloat rtBend = new JSONStorableFloat("Right Thumb Bend", rtBendValue, -1f, 1f, true);
            JSONStorableFloat liBend = new JSONStorableFloat("Left Index Finger Bend", liBendValue, -1f, 1f, true);
            JSONStorableFloat riBend = new JSONStorableFloat("Right Index Finger Bend", riBendValue, -1f, 1f, true);
            JSONStorableFloat lmBend = new JSONStorableFloat("Left Mid Finger Bend", lmBendValue, -1f, 1f, true);
            JSONStorableFloat rmBend = new JSONStorableFloat("Right Mid Finger Bend", rmBendValue, -1f, 1f, true);
            JSONStorableFloat lrBend = new JSONStorableFloat("Left Ring Finger Bend", lrBendValue, -1f, 1f, true);
            JSONStorableFloat rrBend = new JSONStorableFloat("Right Ring Finger Bend", rrBendValue, -1f, 1f, true);
            JSONStorableFloat lpBend = new JSONStorableFloat("Left Pinky Finger Bend", lpBendValue, -1f, 1f, true);
            JSONStorableFloat rpBend = new JSONStorableFloat("Right Pinky Finger Bend", rpBendValue, -1f, 1f, true);
            JSONStorableFloat ltFist = new JSONStorableFloat("Left Thumb Fist", ltFistValue, -1f, 1f, true);
            JSONStorableFloat rtFist = new JSONStorableFloat("Right Thumb Fist", rtFistValue, -1f, 1f, true);
            JSONStorableFloat ltGrasp = new JSONStorableFloat("Left Thumb Grasp", ltGraspValue, -1f, 1f, true);
            JSONStorableFloat rtGrasp = new JSONStorableFloat("Right Thumb Grasp", rtGraspValue, -1f, 1f, true);
            JSONStorableFloat ltInOut = new JSONStorableFloat("Left Thumb In-Out", ltInOutValue, -1f, 1f, true);
            JSONStorableFloat rtInOut = new JSONStorableFloat("Right Thumb In-Out", rtInOutValue, -1f, 1f, true);
            JSONStorableFloat ltStraight = new JSONStorableFloat("Left Thumb Straighten", ltStraightValue, -1f, 1f, true);
            JSONStorableFloat rtStraight = new JSONStorableFloat("Right Thumb Straighten", rtStraightValue, -1f, 1f, true);

            lhFistSlider = CreateSlider(lhFist, false);
            rhFistSlider = CreateSlider(rhFist, true);
            lhGraspSlider = CreateSlider(lhGrasp, false);
            rhGraspSlider = CreateSlider(rhGrasp, true);
            lhSpreadSlider = CreateSlider(lhSpread, false);
            rhSpreadSlider = CreateSlider(rhSpread, true);
            lhChopSlider = CreateSlider(lhChop, false);
            rhChopSlider = CreateSlider(rhChop, true);
            lhStraightSlider = CreateSlider(lhStraight, false);
            rhStraightSlider = CreateSlider(rhStraight, true);

            lfFistSlider = CreateSlider(lfFist, false);
            rfFistSlider = CreateSlider(rfFist, true);
            lfGraspSlider = CreateSlider(lfGrasp, false);
            rfGraspSlider = CreateSlider(rfGrasp, true);
            lfInOutSlider = CreateSlider(lfInOut, false);
            rfInOutSlider = CreateSlider(rfInOut, true);
            lfSqueezeSlider = CreateSlider(lfSqueeze, false);
            rfSqueezeSlider = CreateSlider(rfSqueeze, true);
            lfStraightSlider = CreateSlider(lfStraight, false);
            rfStraightSlider = CreateSlider(rfStraight, true);

            ltBendSlider = CreateSlider(ltBend, false);
            rtBendSlider = CreateSlider(rtBend, true);
            liBendSlider = CreateSlider(liBend, false);
            riBendSlider = CreateSlider(riBend, true);
            lmBendSlider = CreateSlider(lmBend, false);
            rmBendSlider = CreateSlider(rmBend, true);
            lrBendSlider = CreateSlider(lrBend, false);
            rrBendSlider = CreateSlider(rrBend, true);
            lpBendSlider = CreateSlider(lpBend, false);
            rpBendSlider = CreateSlider(rpBend, true);
            ltFistSlider = CreateSlider(ltFist, false);
            rtFistSlider = CreateSlider(rtFist, true);
            ltGraspSlider = CreateSlider(ltGrasp, false);
            rtGraspSlider = CreateSlider(rtGrasp, true);
            ltInOutSlider = CreateSlider(ltInOut, false);
            rtInOutSlider = CreateSlider(rtInOut, true);
            ltStraightSlider = CreateSlider(ltStraight, false);
            rtStraightSlider = CreateSlider(rtStraight, true); 
        }

        private void ChangeSliders()
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            lhFistSlider.slider.onValueChanged.AddListener(delegate (float lhFistValue)
            {
                morphControl.GetMorphByDisplayName("Left Hand Fist").morphValue = lhFistValue;
            });
            rhFistSlider.slider.onValueChanged.AddListener(delegate (float rhFistValue)
            {
                morphControl.GetMorphByDisplayName("Right Hand Fist").morphValue = rhFistValue;
            });
            lhGraspSlider.slider.onValueChanged.AddListener(delegate (float lhGraspValue)
            {
                morphControl.GetMorphByDisplayName("Left Hand Grasp").morphValue = lhGraspValue;
            });
            rhGraspSlider.slider.onValueChanged.AddListener(delegate (float rhGraspValue)
            {
                morphControl.GetMorphByDisplayName("Right Hand Grasp").morphValue = rhGraspValue;
            });
            lhSpreadSlider.slider.onValueChanged.AddListener(delegate (float lhSpreadValue)
            {
                morphControl.GetMorphByDisplayName("Left Hand Spread").morphValue = lhSpreadValue;
            });
            rhSpreadSlider.slider.onValueChanged.AddListener(delegate (float rhSpreadValue)
            {
                morphControl.GetMorphByDisplayName("Right Hand Spread").morphValue = rhSpreadValue;
            });
            lhChopSlider.slider.onValueChanged.AddListener(delegate (float lhChopValue)
            {
                morphControl.GetMorphByDisplayName("Left Hand Chop").morphValue = lhChopValue;
            });
            rhChopSlider.slider.onValueChanged.AddListener(delegate (float rhChopValue)
            {
                morphControl.GetMorphByDisplayName("Right Hand Chop").morphValue = rhChopValue;
            });
            lhStraightSlider.slider.onValueChanged.AddListener(delegate (float lhStraightValue)
            {
                morphControl.GetMorphByDisplayName("Left Hand Straighten").morphValue = lhStraightValue;
            });
            rhStraightSlider.slider.onValueChanged.AddListener(delegate (float rhStraightValue)
            {
                morphControl.GetMorphByDisplayName("Right Hand Straighten").morphValue = rhStraightValue;
            });          

            lfFistSlider.slider.onValueChanged.AddListener(delegate (float lfFistValue)
            {
                morphControl.GetMorphByDisplayName("Left Fingers Fist").morphValue = lfFistValue;
            });
            rfFistSlider.slider.onValueChanged.AddListener(delegate (float rfFistValue)
            {
                morphControl.GetMorphByDisplayName("Right Fingers Fist").morphValue = rfFistValue;
            });
            lfGraspSlider.slider.onValueChanged.AddListener(delegate (float lfGraspValue)
            {
                morphControl.GetMorphByDisplayName("Left Fingers Grasp").morphValue = lfGraspValue;
            });
            rfGraspSlider.slider.onValueChanged.AddListener(delegate (float rfGraspValue)
            {
                morphControl.GetMorphByDisplayName("Right Fingers Grasp").morphValue = rfGraspValue;
            });
            lfInOutSlider.slider.onValueChanged.AddListener(delegate (float lfInOutValue)
            {
                morphControl.GetMorphByDisplayName("Left Fingers In-Out").morphValue = lfInOutValue;
            });
            rfInOutSlider.slider.onValueChanged.AddListener(delegate (float rfInOutValue)
            {
                morphControl.GetMorphByDisplayName("Right Fingers In-Out").morphValue = rfInOutValue;
            });
            lfSqueezeSlider.slider.onValueChanged.AddListener(delegate (float lfSqueezeValue)
            {
                morphControl.GetMorphByDisplayName("Left Fingers Squeeze").morphValue = lfSqueezeValue;
            });
            rfSqueezeSlider.slider.onValueChanged.AddListener(delegate (float rfSqueezeValue)
            {
                morphControl.GetMorphByDisplayName("Right Fingers Squeeze").morphValue = rfSqueezeValue;
            });
            lfStraightSlider.slider.onValueChanged.AddListener(delegate (float lfStraightValue)
            {
                morphControl.GetMorphByDisplayName("Left Fingers Straighten").morphValue = lfStraightValue;
            });
            rfStraightSlider.slider.onValueChanged.AddListener(delegate (float rfStraightValue)
            {
                morphControl.GetMorphByDisplayName("Right Fingers Straighten").morphValue = rfStraightValue;
            });

            ltBendSlider.slider.onValueChanged.AddListener(delegate (float ltBendValue)
            {
                morphControl.GetMorphByDisplayName("Left Thumb Bend").morphValue = ltBendValue;
            });
            rtBendSlider.slider.onValueChanged.AddListener(delegate (float rtBendValue)
            {
                morphControl.GetMorphByDisplayName("Right Thumb Bend").morphValue = rtBendValue;
            });
            liBendSlider.slider.onValueChanged.AddListener(delegate (float liBendValue)
            {
                morphControl.GetMorphByDisplayName("Left Index Finger Bend").morphValue = liBendValue;
            });
            riBendSlider.slider.onValueChanged.AddListener(delegate (float riBendValue)
            {
                morphControl.GetMorphByDisplayName("Right Index Finger Bend").morphValue = riBendValue;
            });
            lmBendSlider.slider.onValueChanged.AddListener(delegate (float lmBendValue)
            {
                morphControl.GetMorphByDisplayName("Left Mid Finger Bend").morphValue = lmBendValue;
            });
            rmBendSlider.slider.onValueChanged.AddListener(delegate (float rmBendValue)
            {
                morphControl.GetMorphByDisplayName("Right Mid Finger Bend").morphValue = rmBendValue;
            });
            lrBendSlider.slider.onValueChanged.AddListener(delegate (float lrBendValue)
            {
                morphControl.GetMorphByDisplayName("Left Ring Finger Bend").morphValue = lrBendValue;
            });
            rrBendSlider.slider.onValueChanged.AddListener(delegate (float rrBendValue)
            {
                morphControl.GetMorphByDisplayName("Right Ring Finger Bend").morphValue = rrBendValue;
            });
            lpBendSlider.slider.onValueChanged.AddListener(delegate (float lpBendValue)
            {
                morphControl.GetMorphByDisplayName("Left Pinky Finger Bend").morphValue = lpBendValue;
            });
            rpBendSlider.slider.onValueChanged.AddListener(delegate (float rpBendValue)
            {
                morphControl.GetMorphByDisplayName("Right Pinky Finger Bend").morphValue = rpBendValue;
            });
            ltFistSlider.slider.onValueChanged.AddListener(delegate (float ltFistValue)
            {
                morphControl.GetMorphByDisplayName("Left Thumb Fist").morphValue = ltFistValue;
            });
            rtFistSlider.slider.onValueChanged.AddListener(delegate (float rtFistValue)
            {
                morphControl.GetMorphByDisplayName("Right Thumb Fist").morphValue = rtFistValue;
            });
            ltGraspSlider.slider.onValueChanged.AddListener(delegate (float ltGraspValue)
            {
                morphControl.GetMorphByDisplayName("Left Thumb Grasp").morphValue = ltGraspValue;
            });
            rtGraspSlider.slider.onValueChanged.AddListener(delegate (float rtGraspValue)
            {
                morphControl.GetMorphByDisplayName("Right Thumb Grasp").morphValue = rtGraspValue;
            });
            ltInOutSlider.slider.onValueChanged.AddListener(delegate (float ltInOutValue)
            {
                morphControl.GetMorphByDisplayName("Left Thumb In-Out").morphValue = ltInOutValue;
            });
            rtInOutSlider.slider.onValueChanged.AddListener(delegate (float rtInOutValue)
            {
                morphControl.GetMorphByDisplayName("Right Thumb In-Out").morphValue = rtInOutValue;
            });
            ltStraightSlider.slider.onValueChanged.AddListener(delegate (float ltStraightValue)
            {
                morphControl.GetMorphByDisplayName("Left Thumb Straighten").morphValue = ltStraightValue;
            });
            rtStraightSlider.slider.onValueChanged.AddListener(delegate (float rtStraightValue)
            {
                morphControl.GetMorphByDisplayName("Right Thumb Straighten").morphValue = rtStraightValue;
            });
        }

        public override void Init()
        {
            try
            {
                #region Button
                var loadPresetUI = CreateButton("Load Preset");
                loadPresetUI.button.onClick.AddListener(() =>
                {
                    FileManagerSecure.CreateDirectory(_handmorphSavePath);
                    var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_handmorphSavePath);
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, _saveExt, _handmorphSavePath, false, true, false, null, false, shortcuts);
                });

                var savePresetUI = CreateButton("Save Preset", true);
                savePresetUI.button.onClick.AddListener(() =>
                {
                    FileManagerSecure.CreateDirectory(_handmorphSavePath);
                    var fileBrowserUI = SuperController.singleton.fileBrowserUI;
                    fileBrowserUI.SetTitle("Save Hand Morphs preset");
                    fileBrowserUI.fileRemovePrefix = null;
                    fileBrowserUI.hideExtension = false;
                    fileBrowserUI.keepOpen = false;
                    fileBrowserUI.fileFormat = _saveExt;
                    fileBrowserUI.defaultPath = _handmorphSavePath;
                    fileBrowserUI.showDirs = true;
                    fileBrowserUI.shortCuts = null;
                    fileBrowserUI.browseVarFilesAsDirectories = false;
                    fileBrowserUI.SetTextEntry(true);
                    fileBrowserUI.Show(HandleSavePreset);
                    fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + "." + _saveExt;
                    fileBrowserUI.ActivateFileNameField();
                });

                button = CreateButton("Hand Morphs to Default");
                if (button != null)
                {
                    button.button.onClick.AddListener(DefaultHandMorphs);
                }

                Getbutton = CreateButton("Hand Morphs to Zero", true);
                if (Getbutton != null)
                {
                    Getbutton.button.onClick.AddListener(ZeroHandMorps);
                }                

                clbutton = CreateButton("Copy Left to Right", false);
                if (clbutton != null)
                {
                    clbutton.button.onClick.AddListener(CopyLtoR);
                }

                crbutton = CreateButton("Copy Right to Left", true);
                if (crbutton != null)
                {
                    crbutton.button.onClick.AddListener(CopyRtoL);
                }

                swbutton = CreateButton("Swap Left and Right", false);
                if (swbutton != null)
                {
                    swbutton.button.onClick.AddListener(SwapLandR);
                }

                toggle_x = new JSONStorableBool("Enable swap loading Preset", false);
                CreateToggle(toggle_x, true);

                toggle_l = new JSONStorableBool("Enable loading Preset to Left", true);
                CreateToggle(toggle_l, false);
                toggle_r = new JSONStorableBool("Enable loading Preset to Right", true);
                CreateToggle(toggle_r, true);

                #endregion

                #region Sliders
                    GetSliderValue();
                    CreateSliders();
                    ChangeSliders(); 
                #endregion
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }       
        }

        #region Preset Load and Save
        private void HandleLoadPreset(string path)
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;
            LoadData = SuperController.singleton.ReadFileIntoString(path);
            //  SuperController.LogMessage(s);
            string[] LoadDataList = LoadData.Split('\n');
            int i=0;
            foreach (string HandMorphName in LoadDataList)
            {
                string[] NameAndValue = LoadDataList[i].Split(',');
                i = i + 1;
                //SuperController.LogMessage(mv[0]);
                //SuperController.LogMessage(mv[1]);
                int len = NameAndValue[0].Length;

                if (toggle_x.val)
                {
                    if (NameAndValue[0].Substring(0, 5) == "Right")
                    {
                        NameAndValue[0] = "Left " + NameAndValue[0].Substring(6, len - 6);
                        //SuperController.LogMessage("Left " + NameAndValue[0].Substring(6, len - 6));
                    }
                    else
                    {
                        NameAndValue[0] = "Right " + NameAndValue[0].Substring(5, len  - 5);
                        //SuperController.LogMessage("Right " + NameAndValue[0].Substring(5, len - 5));
                    }
                }

                //SuperController.LogMessage(NameAndValue[0]);

                if (toggle_l.val && NameAndValue[0].Substring(0, 1) == "L")
                {
                    morphControl.GetMorphByDisplayName(NameAndValue[0]).morphValue = float.Parse(NameAndValue[1]);
                }
                
                if (toggle_r.val && NameAndValue[0].Substring(0, 1) == "R")
                {
                    morphControl.GetMorphByDisplayName(NameAndValue[0]).morphValue = float.Parse(NameAndValue[1]);
                }
            }
        }

        private void HandleSavePreset(string path)
        {
            JSONStorable geometry = containingAtom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            List<string> HandMorphList = new List<string>(HandMorphs);
            foreach (string HandMorphs in HandMorphList)
            {
                SaveData = SaveData + "Right " + HandMorphs + "," + morphControl.GetMorphByDisplayName("Right "+HandMorphs).morphValue.ToString() + "\n";
                SaveData = SaveData + "Left " + HandMorphs + "," + morphControl.GetMorphByDisplayName("Left "+HandMorphs).morphValue.ToString() + "\n";
            }
            //  SuperController.LogMessage(s);
            if (path.Substring(path.Length-10,10) != ".handmorph")
            {
                path = path + ".handmorph";
            }
            //SuperController.LogMessage(path.Substring(path.Length-10,10));
            SuperController.singleton.SaveStringIntoFile(path, SaveData);
        }
        #endregion

        protected UIDynamicButton button;
        protected UIDynamicButton Getbutton;        
        protected UIDynamicButton clbutton;
        protected UIDynamicButton crbutton; 
        protected UIDynamicButton swbutton; 
        protected UIDynamicButton loadbutton;   
        protected UIDynamicButton savebutton;   

        public UIDynamicSlider ltBendSlider;
        public UIDynamicSlider rtBendSlider;
        public UIDynamicSlider liBendSlider;
        public UIDynamicSlider riBendSlider;
        public UIDynamicSlider lmBendSlider;
        public UIDynamicSlider rmBendSlider;
        public UIDynamicSlider lrBendSlider;
        public UIDynamicSlider rrBendSlider;
        public UIDynamicSlider lpBendSlider;
        public UIDynamicSlider rpBendSlider;
        public UIDynamicSlider ltFistSlider;
        public UIDynamicSlider rtFistSlider;
        public UIDynamicSlider ltGraspSlider;
        public UIDynamicSlider rtGraspSlider;
        public UIDynamicSlider ltInOutSlider;
        public UIDynamicSlider rtInOutSlider;
        public UIDynamicSlider ltStraightSlider;
        public UIDynamicSlider rtStraightSlider;

        public UIDynamicSlider lfFistSlider;
        public UIDynamicSlider rfFistSlider;
        public UIDynamicSlider lfGraspSlider;
        public UIDynamicSlider rfGraspSlider;
        public UIDynamicSlider lfInOutSlider;
        public UIDynamicSlider rfInOutSlider;
        public UIDynamicSlider lfSqueezeSlider;
        public UIDynamicSlider rfSqueezeSlider;
        public UIDynamicSlider lfStraightSlider;
        public UIDynamicSlider rfStraightSlider;

        public UIDynamicSlider lhFistSlider;
        public UIDynamicSlider rhFistSlider;
        public UIDynamicSlider lhGraspSlider;
        public UIDynamicSlider rhGraspSlider;
        public UIDynamicSlider lhSpreadSlider;
        public UIDynamicSlider rhSpreadSlider;
        public UIDynamicSlider lhChopSlider;
        public UIDynamicSlider rhChopSlider;
        public UIDynamicSlider lhStraightSlider;
        public UIDynamicSlider rhStraightSlider;

	    // Use this for initialization
//       public void ResetHandMorpButtonCallback ()
//        {
//            ResetHandMorphs();
//            UpdateSliders();
//        }

        public void Start()
        {
//            GetValue();
        }

       public void Update()
        {
            GetValue();
        }

    }

}

